%% MATLAB Code of Trauth et al., Quaternary Science Reviews, 2021
%
% Trauth, M.H., Asrat, A., Cohen, A., Duesing, W., Foerster, V.,
% Kaboth-Bahr, S., Kraemer, H.,  Lamb, H., Marwan, N., Maslin, M.,
% Schaebitz, F. (2021) Recurring types of variability and transitions in
% the ~620 kyr record of climate change from the Chew Bahir basin, southern
% Ethiopia, Quaternary Science Reviews.
%
% https://doi.org/10.1016/j.quascirev.2020.106777
%
% Required functions and external scripts:
% - script_pretreatment_age.m
% - script_noisefilter_age.m
% - script_recurrenceplot_age.m
% - script_recurrencequantificationanalysis_age.m
% - script_displayresults_rqa_age.m
% - embed.m
% - nonlin_noise_reduction.m
% - rp.m
% - rqa.m
%
% Copyright (c) 2021
%
% Martin H. Trauth, University of Potsdam, Potsdam, Germany
% http://www.martinhtrauth.de
%
% Contact: trauth@uni-potsdam.de
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or any later version.

clear, clc, close all

% Load data.
data = load('data_potassium_age.txt');

% Define time and data vector.
t = data(:,1); t = t';
x = data(:,2); x = x';

% Graphics properties.
xmin =  -650;
xmax =  0;
ymin = 3e4;
ymax = 13e4;
ydir = 'Reverse';      

% Optional filter to remove trend.
filteroption = 0;
filterorder  = 5;
filtercutoff = 0.1;

% Define embedding dimension m and time delay tau.
m = 6;
tau = 10;
timespan_diff = tau*(m-1);

% Define Theiler window and minimal line length.
theiler = tau;
l_min = 10;

% Define window size (w) and step size (ws).
w  = 500;
ws = 50;

% Set correction for border effects on (line_correct = 1) or off
% (line_correct = 0)
line_correct = 1;

% Select one RQA measures, which will be displayed in a separate figure
% together with RR and DET in the first window
%     Y(1) = RR     (recurrence rate)
%     Y(2) = DET    (determinism)
%     Y(3) = <L>    (mean diagonal line length)
%     Y(4) = Lmax   (maximal diagonal line length)
%     Y(5) = ENTR   (entropy of the diagonal line lengths)
%     Y(6) = LAM    (laminarity)
%     Y(7) = TT     (trapping time)
%     Y(8) = Vmax   (maximal vertical line length)
%     Y(9) = RTmax  (maximal white vertical line length)
%     Y(10) = T2    (recurrence time of 2nd type)
%     Y(11) = RTE   (recurrence time entropy, i.e., RPDE)
%     Y(12) = Clust (clustering coefficient)
%     Y(13) = Trans (transitivity)
RQA_Select = 1;

% Calculate recurrence plot using the recurrence threshold e and choose
% threshold-calculation parameter. Set parameter to define the threshold-
% calculation for the recurrence plot estimation in the next step. There
% are three options to choose from:
%   'fix'  The RP is computed under a fixed threshold e specified by input
%          parameter e.
%   'var'  The RP is computed under a fixed threshold e, which corresponds
%          to the lower e-quantile (specified by input parameter e of the 
%          distance distribution of all points in phase space.
%   'fan'  The RP is computed under a variable threshold e using a fixed
%          amount of nearest neighbours in phase space to compute the e-
%          value for each point of the phase space trajectory.
threshold_calculation = 'fan';

% Choose norm from ['euc','max']
norm = 'euc';

% Data pretreatment, compute RQ, perform RQA and display results.
script_pretreatment_age
script_noisefilter_age
script_recurrenceplot_age
script_recurrencequantificationanalysis_age
script_displayresults_rqa_age


