%% Script to display RQ/RQA results
%
% Trauth, M.H., Asrat, A., Cohen, A., Duesing, W., Foerster, V.,
% Kaboth-Bahr, S., Kraemer, H.,  Lamb, H., Marwan, N., Maslin, M.,
% Schaebitz, F. (2021) Recurring types of variability and transitions in
% the ~620 kyr record of climate change from the Chew Bahir basin, southern
% Ethiopia, Quaternary Science Reviews.
%
% https://doi.org/10.1016/j.quascirev.2020.106777
%
% Required functions and external scripts:
% (none)
%
% Expected variables:
% - xmin, xmax (scalars to specify the x-axis limits 
% - ymin, ymax (scalars to specify the y-axis limits
% - ydir (string to reverse the y-axis)    
% - r_win_e (vector with the results of RQA)
% - m (scalar with embedding dimension)
% - tau (scalar with embedding delay)
% - threshold_calculation (string with the recurrence criterion)
% - norm (string with used norm for distance calculation)
% - w (scalar specifying the window size)
% - ws (scalar that sets the window step width)
% - l_min (scalar of minimum line length)
% - theiler (scalar with size of the Theiler correction window)
% - RQA_Select (scalar with the index of the RQA variable)
%
% Delivers:
% (figures with results)
%
% Copyright (c) 2021
%
% Martin H. Trauth, University of Potsdam, Potsdam, Germany
% http://www.martinhtrauth.de
%
% Contact: trauth@uni-potsdam.de
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or any later version.

RR(isnan(RR)==1) = 0;

RQA_Legend = {
     'RR'
     'DET'
     '<L>'
     'Lmax'
     'ENTR'
     'LAM'
     'TT'
     'Vmax'
     'RTmax'
     'T2'
     'RTE'
     'Clust'
     'Trans'};

figure1 = figure('Units','normalized',...
    'Position',[.1 .1 .4 .9],...
    'Color',[1 1 1]);

axes1(1) = axes('Units','normalized',...
    'Position',[0.1 0.82 0.7 0.15],...
    'XLim',[xmin xmax],...
    'YLim',[ymin ymax],...
    'YDir',ydir,...
    'LineWidth',1.5,...
    'Box','On',...
    'XGrid','On',...
    'XTickLabel',''); hold on
line(t,x,...
    'LineWidth',1,...
    'Color',[0 0 0])
title('Chew Bahir Potassium Concentration')

axes1(2) = axes('Units','normalized',...
    'Position',[0.1 0.16 0.7 0.7],...
    'XLim',[xmin xmax],...
    'YLim',[xmin xmax],...
    'LineWidth',1.5,...
    'Box','On',...
    'Layer','top',...
    'XGrid','On',...
    'XTickLabel','',...
    'YTickLabel',''); hold on
axis square xy
imagesc(t,t,RR)
colormap([1 1 1; 0 0 0])

if strcmp(threshold_calculation,'var')
    e = round(eps,2);
end

str1 = ['m    = ',num2str(m)];
str2 = ['tau  = ',num2str(tau)];
if strcmp(threshold_calculation,'fix')
    str3 = ['e    = ',num2str(e),' (fixed)'];
elseif strcmp(threshold_calculation,'var')
    str3 = ['e    = ',num2str(e),' (fixed-var)'];
elseif strcmp(threshold_calculation,'fan')
    str3 = ['e    = ',num2str(e),' (adaptive)'];
end
str4 = ['w    = ',num2str(w)];
str5 = ['ws   = ',num2str(ws)];
str6 = ['norm = ',norm];
str7 = ['thei = ',num2str(theiler)];
str8 = ['lmin = ',num2str(l_min)];
str = {str1,str2,str3,str4,str5,str6,str7,str8};
text(xmin+(xmax-xmin)/20,...
     xmax-(xmax-xmin)/20,str,...
    'BackgroundColor',[1 1 1],...
    'EdgeColor',[1 1 1],...
    'VerticalAlignment','top')

axes1(3) = axes('Units','normalized',...
    'Position',[0.1 0.05 0.7 0.15],...
    'XLim',[xmin xmax],...
    'LineWidth',1.5,...
    'XGrid','On',...
    'Box','On'); hold on
line(axes1(3),t,r_win_e(RQA_Select(1),:),...
    'LineWidth',1,...
    'Color',[0 0.4453 0.7383])
legend(RQA_Legend{RQA_Select(1)},...
    'Box','Off')
xlabel('Age (kyr BP)')

linkaxes(axes1,'x')
set(axes1(2),'tag','rp2')

figstr = [
    'hh=findobj(figure1,''tag'',''rp2'');',...
    'xx = get(hh,''xlim'');'...
    'set(hh,''ylim'',xx)'];

set(figure1,'WindowButtonMotionFcn',figstr)

